use cpal::Sample;

const TAU: f64 = 2.0 * std::f64::consts::PI;

pub struct TwoStateCarrier {
    pub sample_rate: u32,
    pub freq_low: u32,
    pub freq_high: u32,
    phase_low: u32,
    phase_high: u32,
}

type PcmSample = i64;

pub trait Carrier {
    fn byte_into_fsk_samples(&mut self, byte: &u8) -> Vec<PcmSample>;
}

impl Carrier for TwoStateCarrier {
    fn byte_into_fsk_samples(&mut self, byte: &u8) -> Vec<PcmSample> {
        let mut bits: Vec<u8> = Vec::new();
        let mut out: Vec<i64> = Vec::new();

        // MSb first
        bits.push((byte     ) & 0b0000_0001);
        bits.push((byte >> 1) & 0b0000_0010);
        bits.push((byte >> 2) & 0b0000_0100);
        bits.push((byte >> 3) & 0b0000_1000);
        bits.push((byte >> 4) & 0b0001_0000);
        bits.push((byte >> 5) & 0b0010_0000);
        bits.push((byte >> 6) & 0b0100_0000);
        bits.push((byte >> 7) & 0b1000_0000);

        for bit in bits {
            if bit == 0 {
                while self.phase_low < self.sample_rate / self.freq_low {
                    out.push((self.phase_low as f64 * TAU * self.freq_low as f64 / self.sample_rate as f64).sin().to_sample::<i64>());
                    self.phase_low += 1;
                }
                self.phase_low = self.phase_low % self.sample_rate;
            } else if bit == 1 {
                while self.phase_high < self.sample_rate / self.freq_high {
                    out.push((self.phase_high as f64 * TAU * self.freq_low as f64 / self.sample_rate as f64).sin().to_sample::<i64>());
                    self.phase_high += 1;
                }
                self.phase_high = self.phase_high % self.sample_rate;
            }
        }
        out
    }
}

impl TwoStateCarrier {
    pub fn new(freq_low: u32, freq_high: u32, sample_rate: u32) -> Self {
        TwoStateCarrier {
            freq_low: freq_low,
            freq_high: freq_high,
            sample_rate: sample_rate,
            phase_low: 0,
            phase_high: 0,
        }
    }
    fn freq_low(mut self, freq_low: u32) {
        self.freq_low = freq_low;
    }
    fn freq_high(mut self, freq_high: u32) {
        self.freq_high = freq_high;
    }
    fn phase_low(mut self, phase_low: u32) {
        self.phase_low = phase_low;
    }
    fn phase_high(mut self, phase_high: u32) {
        self.phase_high = phase_high;
    }
}
